@@ -34,7 +34,7 @@ import traceback
 from collections import Callable
 from tomorrow3 import threads as tomorrow_threads
 
-from redis_queue_tool.utils import str_sha256, get_now_millseconds, get_day_formate
+from redis_queue_tool.utils import str_sha256, get_now_millseconds
 
 logger = get_logger(__name__, formatter_template=5)
 
@@ -66,12 +66,11 @@ def init_redis_config(host, password, port, db):
 class RedisCustomer(object):
     """reids队列消费类"""
 
-    process_ids = []
-
     def __init__(self, queue_name, consuming_function: Callable = None, process_num=1, threads_num=50,
                  max_retry_times=3, func_timeout=None, is_support_mutil_param=True, qps=0,
                  middleware=MiddlewareEum.REDIS,
-                 specify_threadpool=None, customer_type='thread', fliter_rep=False, max_push_size=50, ack=False):
+                 specify_threadpool=None, customer_type='thread', fliter_rep=False, max_push_size=50, ack=False,
+                 priority=None):
         """
         redis队列消费程序
         :param queue_name: 队列名称
@@ -87,6 +86,7 @@ class RedisCustomer(object):
         :param fliter_rep: 消费任务是否去重 bool True:去重 False:不去重
         :param max_push_size : 每次批量推送任务数量 默认值50
         :param ack : 是否需要确认消费 默认值False
+        :param priority : 队列优先级 int[0-4]
         """
         if middleware == MiddlewareEum.SQLITE:
             self._redis_quenen = SqlliteQueue(queue_name=queue_name)
@@ -96,7 +96,8 @@ class RedisCustomer(object):
         elif middleware == MiddlewareEum.MEMORY:
             self._redis_quenen = MemoryQueue(queue_name=queue_name)
         else:
-            self._redis_quenen = RedisQueue(queue_name, host=default_config.redis_host, port=default_config.redis_port,
+            self._redis_quenen = RedisQueue(queue_name, priority=priority, host=default_config.redis_host,
+                                            port=default_config.redis_port,
                                             db=default_config.redis_db,
                                             password=default_config.redis_password)
         self._consuming_function = consuming_function
@@ -161,6 +162,10 @@ class RedisCustomer(object):
                 else:
                     time.sleep(0.3)
             except KeyboardInterrupt:
+                heartbeat_check_queue_name = f"_heartbeat_check_common:{self._redis_quenen.queue_name}"
+                if self._redis_quenen.getdb().setnx(heartbeat_check_queue_name, 1):
+                    self._redis_quenen.getdb().expire(heartbeat_check_queue_name, 10)
+                    self._heartbeat_check_common(is_break=True)
                 self._clear_process()
             except:
                 logger.error(traceback.format_exc())
@@ -179,8 +184,6 @@ class RedisCustomer(object):
                     f'start consuming message  process:{i + 1},{self.customer_type}_num:{self.threads_num},system:{platform.system()}')
                 p = Process(target=self._start_consuming_message_thread)
                 p.start()
-                self.process_ids.append(p.pid)
-                logger.info(self.process_ids)
         else:
             logger.info(
                 f'start consuming message {self.customer_type}, {self.customer_type}_num:{self.threads_num},system:{platform.system()}')
@@ -204,6 +207,11 @@ class RedisCustomer(object):
                 self._redis_quenen.ack(message)
         except:
             logger.error(traceback.format_exc())
+            if self.ack:
+                if self.middleware == MiddlewareEum.REDIS:
+                    dlq_message = json.dumps(message) if isinstance(message, dict) else message
+                    self._redis_quenen.getdb().lpush(f'dlq:{self._redis_quenen.queue_name}', dlq_message)
+            logger.warning('consuming_exception_retry faliture')
         else:
             if self.fliter_rep:
                 if self.middleware == MiddlewareEum.REDIS:
@@ -218,32 +226,39 @@ class RedisCustomer(object):
             logger.info('启动redis心跳检查程序')
             while True:
                 try:
-                    if self._redis_quenen.getdb().hexists(self._redis_quenen.heartbeat_key,
-                                                          self._redis_quenen.heartbeat_field) is False:
-                        self._redis_quenen.getdb().hset(self._redis_quenen.heartbeat_key,
-                                                        self._redis_quenen.heartbeat_field, get_now_millseconds())
-                    hash_all_data = self._redis_quenen.getdb().hgetall(self._redis_quenen.heartbeat_key)
-                    if hash_all_data:
-                        for key in hash_all_data:
-                            heart_field = key.decode()
-                            heart_value = int(hash_all_data[key].decode())
-                            if get_now_millseconds() - heart_value > 10500:
-                                queue_name = heart_field.split(':heartbeat_')[0]
-                                un_ack_sets_name = f"{heart_field}:unack_message"
-                                for i in self._redis_quenen.getdb().sscan_iter(un_ack_sets_name):
-                                    self._redis_quenen.getdb().lpush(queue_name, i.decode())
-                                    self._redis_quenen.getdb().srem(un_ack_sets_name, i)
-                                self._redis_quenen.getdb().hdel(self._redis_quenen.heartbeat_key, heart_field)
-                            else:
-                                self._redis_quenen.getdb().hset(self._redis_quenen.heartbeat_key,
-                                                                heart_field,
-                                                                get_now_millseconds())
+                    self._heartbeat_check_common()
+                except KeyboardInterrupt:
+                    logger.info('redis队列心跳检查程序退出')
                 except Exception as ex:
                     logger.error(ex)
                     time.sleep(10)
                 else:
                     time.sleep(10)
 
+    def _heartbeat_check_common(self, is_break=False):
+        logger.info('_heartbeat_check_common')
+        if self._redis_quenen.getdb().hexists(self._redis_quenen.heartbeat_key,
+                                              self._redis_quenen.heartbeat_field) is False:
+            self._redis_quenen.getdb().hset(self._redis_quenen.heartbeat_key,
+                                            self._redis_quenen.heartbeat_field, get_now_millseconds())
+        hash_all_data = self._redis_quenen.getdb().hgetall(self._redis_quenen.heartbeat_key)
+        if hash_all_data:
+            for key in hash_all_data:
+                heart_field = key.decode()
+                heart_value = int(hash_all_data[key].decode())
+                if is_break or get_now_millseconds() - heart_value > 10500:
+                    queue_name = heart_field.split(':heartbeat_')[0]
+                    un_ack_sets_name = f"{heart_field}:unack_message"
+                    dlq_queue_name = f"dlq:{queue_name}"
+                    for i in self._redis_quenen.getdb().sscan_iter(un_ack_sets_name):
+                        self._redis_quenen.getdb().lpush(dlq_queue_name, i.decode())
+                        self._redis_quenen.getdb().srem(un_ack_sets_name, i)
+                    self._redis_quenen.getdb().hdel(self._redis_quenen.heartbeat_key, heart_field)
+                else:
+                    self._redis_quenen.getdb().hset(self._redis_quenen.heartbeat_key,
+                                                    heart_field,
+                                                    get_now_millseconds())
+
     def _clear_process(self):
         pid = os.getpid()
         logger.warning(f'{pid} process is KeyboardInterrupt and kill')
@@ -253,16 +268,20 @@ class RedisCustomer(object):
 class RedisPublish(object):
     """redis入队列类"""
 
-    def __init__(self, queue_name, fliter_rep=False, max_push_size=50, middleware=MiddlewareEum.REDIS,
+    def __init__(self, queue_name, fliter_rep=False, priority: int = None, max_push_size=50,
+                 middleware=MiddlewareEum.REDIS,
                  consuming_function: Callable = None):
         """
         初始化消息发布队列
         :param queue_name: 队列名称(不包含命名空间)
         :param fliter_rep: 队列任务是否去重 True:去重  False:不去重
+        :param priority: 队列优先级
         :param max_push_size: 使用批量提交时,每次批量提交数量
         :param middleware: 中间件,默认redis 支持sqlite,kafka
         :param consuming_function: 消费函数名称
         """
+        if priority and priority > 4:
+            raise Exception('max priority support is 4')
         if middleware == MiddlewareEum.SQLITE:
             self._redis_quenen = SqlliteQueue(queue_name=queue_name)
         elif middleware == MiddlewareEum.KAFKA:
@@ -271,7 +290,7 @@ class RedisPublish(object):
         elif middleware == MiddlewareEum.MEMORY:
             self._redis_quenen = MemoryQueue(queue_name=queue_name)
         else:
-            self._redis_quenen = RedisQueue(queue_name, host=default_config.redis_host,
+            self._redis_quenen = RedisQueue(queue_name, priority=priority, host=default_config.redis_host,
                                             port=default_config.redis_port,
                                             db=default_config.redis_db,
                                             password=default_config.redis_password)
@@ -367,7 +386,7 @@ class RedisPublish(object):
         if self._local_quenen.qsize() >= self.max_push_size:
             try:
                 while self._local_quenen.qsize() > 0:
-                    self._pipe.lpush(self._redis_quenen.key, self._local_quenen.get_nowait())
+                    self._pipe.lpush(self._redis_quenen.queue_name, self._local_quenen.get_nowait())
             except:
                 logger.error(traceback.format_exc())
             self._pipe.execute()
@@ -387,15 +406,37 @@ class RedisPublish(object):
         """
         return self._redis_quenen.qsize()
 
+    def dlq_re_queue(self):
+        if self.middleware == MiddlewareEum.REDIS:
+            self._redis_quenen.dlq_re_consume()
+        else:
+            pass
 
-def task_deco(queue_name, process_num=1, threads_num=50,
+
+def task_deco(queue_name,
+              priority=None,
+              process_num=1,
+              threads_num=50,
               max_retry_times=3, qps=0,
               middleware=MiddlewareEum.REDIS,
-              specify_threadpool=None, customer_type='thread', fliter_rep=False, ack=False, *consumer_args,
+              specify_threadpool=None,
+              customer_type='thread',
+              fliter_rep=False,
+              ack=False, *consumer_args,
               **consumer_init_kwargs):
     """
     support by ydf
     装饰器方式添加任务，如果有人过于喜欢装饰器方式，例如celery 装饰器方式的任务注册，觉得黑科技，那就可以使用这个装饰器。此种方式不利于ide代码自动补全不推荐。
+    :param priority:
+    :param process_num:
+    :param threads_num:
+    :param max_retry_times:
+    :param qps:
+    :param middleware:
+    :param specify_threadpool:
+    :param customer_type:
+    :param fliter_rep:
+    :param ack:
     :param queue_name:
     :param consumer_init_kwargs:
     :return:
@@ -418,7 +459,7 @@ def task_deco(queue_name, process_num=1, threads_num=50,
     def _deco(func):
         cs = RedisCustomer(queue_name, process_num=process_num, threads_num=threads_num, middleware=middleware, qps=qps,
                            consuming_function=func, specify_threadpool=specify_threadpool, customer_type=customer_type,
-                           fliter_rep=fliter_rep, ack=ack, max_retry_times=max_retry_times,
+                           fliter_rep=fliter_rep, ack=ack, max_retry_times=max_retry_times, priority=priority,
                            *consumer_args, **consumer_init_kwargs)
         if 'consuming_function' in consumer_init_kwargs:
             consumer_init_kwargs.pop('consuming_function')
@@ -426,7 +467,7 @@ def task_deco(queue_name, process_num=1, threads_num=50,
         # 下面这些连等主要是由于元编程造成的不能再ide下智能补全，参数太长很难手动拼写出来
         func.start_consuming_message = func.consume = func.start = cs.start_consuming_message
 
-        publisher = RedisPublish(queue_name=queue_name, consuming_function=cs._consuming_function,
+        publisher = RedisPublish(queue_name=queue_name, priority=priority, consuming_function=cs._consuming_function,
                                  fliter_rep=cs.fliter_rep, max_push_size=cs.max_push_size, middleware=cs.middleware)
         func.publisher = publisher
         func.pub = func.publish = func.publish_redispy = publisher.publish_redispy
@@ -443,18 +484,22 @@ def task_deco(queue_name, process_num=1, threads_num=50,
 
 if __name__ == '__main__':
     # #装饰器使用方式
-    @task_deco('test1', process_num=3)  # 消费函数上新增任务队列装饰器
+    @task_deco('test12', priority=1, process_num=3, ack=True)  # 消费函数上新增任务队列装饰器
     def f(a, b):
         print(f"a:{a},b:{b}")
+        # raise Exception('test1 exception')
+        time.sleep(1)
 
 
     # 发布任务
-    for i in range(1, 51):
-        f.pub(a=1, b=1)
+    # for i in range(1, 200):
+    #     f.pub(a=i, b=i)
 
     # 消费任务
     f.start()
 
+    # f.publisher.dlq_re_queue()
+
     # # #非装饰器版使用demo
     # for zz in range(1, 51):
     #     RedisPublish(queue_name='test2').pub(a=zz, b=zz, c=zz)
